import platform
import psutil

from addr_agent.version import __version__


def bytes_to_gb(b):
    gb = b/(1024 * 1024 * 1024)
    gb = round(gb, 2)
    return gb


def generate_hw_report() -> str:
    report = f"Addr Agent version: {__version__}\n\n"

    report += f"Architecture: {platform.architecture()[0]}\n"
    report += f"Machine: {platform.machine()}\n"
    report += f"Operating System Release: {platform.release()}\n"
    report += f"Operating System Version: {platform.version()}\n"
    report += f"Platform: {platform.platform()}\n\n"

    report += f"Number of Physical cores: {str(psutil.cpu_count(logical=False))}\n"
    report += f"Number of Total cores: {str(psutil.cpu_count(logical=True))}\n"

    with open("/proc/cpuinfo", "r") as f:
        file_info = f.readlines()

    cpuinfo = [x.strip().split(":")[1] for x in file_info if "model name" in x]
    for index, item in enumerate(cpuinfo):
        report += f"Processor {str(index)}: {item}\n"

    virtual_memory = psutil.virtual_memory()
    report += f"Total Memory Present: {bytes_to_gb(virtual_memory.total)}Gb\n"

    swap = psutil.swap_memory()
    report += f"Total swap memory: {bytes_to_gb(swap.total)}Gb\n"

    return report
