import json
from typing import Generator

import aiodocker

from addr_agent.version import __version__
from addr_agent.agent_feature.models import RPCMethodImpl, RPCTaskResult, RPCTaskStatus


class RPCAddrV1AgentUpdateMethod(RPCMethodImpl):
    async def run(self) -> Generator[RPCTaskResult, None, None]:
        request = json.loads(self.request)
        version = request["version"]

        yield RPCTaskResult(
            status=RPCTaskStatus.PROGRESS,
            response=f"Current version {__version__}, new version {version}"
        )
        if version == __version__:
            yield RPCTaskResult(status=RPCTaskStatus.PROGRESS, response="No need in update")
            yield RPCTaskResult(status=RPCTaskStatus.COMPLETED)
            return

        self.agent_feature.fetch_tasks = False

        docker = aiodocker.Docker()
        yield RPCTaskResult(status=RPCTaskStatus.PROGRESS, response="Pulling new addr-runner container")
        response = await docker.images.pull("registry.uwtech.org/uwtech/cdn/addr-agent:latest")
        for resp in response:
            yield RPCTaskResult(status=RPCTaskStatus.PROGRESS, print_datetime=False, response=resp["status"])

        yield RPCTaskResult(status=RPCTaskStatus.PROGRESS, response="Pulling intermediate updater container")
        response = await docker.images.pull("docker:latest")
        for resp in response:
            yield RPCTaskResult(status=RPCTaskStatus.PROGRESS, print_datetime=False, response=resp["status"])

        yield RPCTaskResult(status=RPCTaskStatus.PROGRESS, response="Start updater container")
        yield RPCTaskResult(status=RPCTaskStatus.COMPLETED)
        await docker.containers.run(
            config={
                "Image": "docker:latest",
                "Cmd": [
                    "/bin/sh",
                    "-c",
                    "sleep 10 && (docker stop addr-agent || true) && (docker remove addr-agent || true) && "
                    "docker run -d --name addr-agent --network host --restart unless-stopped "
                    "--volume /var/run/docker.sock:/var/run/docker.sock:rw "
                    "--volume /:/host:rw "
                    f"--env 'APP__AGENT__HOST={self.agent_feature.config['host']}' "
                    f"--env 'APP__AGENT__TOKEN={self.agent_feature.config['token']}' "
                    "registry.uwtech.org/uwtech/cdn/addr-agent:latest",
                ],
                "HostConfig": {
                    "AutoRemove": True,
                    "Binds": [
                        "/var/run/docker.sock:/var/run/docker.sock:rw",
                    ],
                }
            },
            name="addr-agent-updater",
        )

        self.agent_feature.app.shutdown()
