import asyncio
import json
from typing import Generator

import routeros_api
from addr_agent.agent_feature.models import RPCMethodImpl, RPCTaskResult, RPCTaskStatus


class RPCAddrV1FtUnblockUpdateMethod(RPCMethodImpl):
    async def run(self) -> Generator[RPCTaskResult, None, None]:
        request = json.loads(self.request)
        address = request["address"]
        user = request["user"]
        password = request["password"]
        address_list = request["address_list"]
        routes = request["routes"]

        yield RPCTaskResult(
            status=RPCTaskStatus.PROGRESS,
            response=f"Update address list {address_list} for {address}",
        )

        connection = routeros_api.RouterOsApiPool(
            address,
            username=user,
            password=password,
            port=8728,
            plaintext_login=True,
        )
        api = connection.get_api()

        address_lists = api.get_resource("/ip/firewall/address-list")
        for route in routes:
            yield RPCTaskResult(
                status=RPCTaskStatus.PROGRESS,
                response=f"Add route {route} to {address_list}",
            )
            entry = address_lists.get(list=address_list, address=route)
            if not entry:
                address_lists.add(
                    list=address_list,
                    address=route,
                    comment="Added by UW Addr",
                )
            await asyncio.sleep(0.01)

        connection.disconnect()

        yield RPCTaskResult(status=RPCTaskStatus.COMPLETED)
