import asyncio
import json
import pathlib
from typing import Generator

from addr_agent.agent_feature.models import RPCMethodImpl, RPCTaskResult, RPCTaskStatus


class RPCAddrV1NebulaCreateCaMethod(RPCMethodImpl):
    async def run(self) -> Generator[RPCTaskResult, None, None]:
        request = json.loads(self.request)
        ca_name = request["ca_name"]
        duration = request["duration"]

        public_key_file = pathlib.Path("ca.crt")
        private_key_file = pathlib.Path("ca.key")
        public_key_file.unlink(missing_ok=True)
        private_key_file.unlink(missing_ok=True)

        proc = await asyncio.create_subprocess_shell(
            f"nebula-cert ca -name '{ca_name}' -duration {duration}",
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE)
        stdout, stderr = await proc.communicate()

        exit_code = await proc.wait()
        if exit_code == 0:
            response = {
                "ca_public_key": public_key_file.read_text(),
                "ca_private_key": private_key_file.read_text(),
            }
            public_key_file.unlink()
            private_key_file.unlink()
            yield RPCTaskResult(status=RPCTaskStatus.COMPLETED, response=json.dumps(response))
        else:
            yield RPCTaskResult(
                status=RPCTaskStatus.ERROR, response=f"Process exited with code {exit_code}\n: {stderr.decode('utf-8')}"
            )
