import asyncio
import json
import pathlib
from typing import Generator

from addr_agent.agent_feature.models import RPCMethodImpl, RPCTaskResult, RPCTaskStatus


class RPCAddrV1NebulaCreateHostMethod(RPCMethodImpl):
    async def run(self) -> Generator[RPCTaskResult, None, None]:
        request = json.loads(self.request)
        ca_public_key = request["ca_public_key"]
        ca_private_key = request["ca_private_key"]
        host_name = request["host_name"]
        host_ip = request["host_ip"]

        public_key_file = pathlib.Path("ca.crt")
        private_key_file = pathlib.Path("ca.key")
        host_public_key_file = pathlib.Path(host_name + ".crt")
        host_private_key_file = pathlib.Path(host_name + ".key")
        public_key_file.write_text(ca_public_key)
        private_key_file.write_text(ca_private_key)
        host_public_key_file.unlink(missing_ok=True)
        host_private_key_file.unlink(missing_ok=True)

        proc = await asyncio.create_subprocess_shell(
            f"nebula-cert sign -name '{host_name}' -ip {host_ip}",
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE
        )
        stdout, stderr = await proc.communicate()

        exit_code = await proc.wait()
        if exit_code == 0:
            response = {
                "host_name": int(host_name),
                "host_public_key": host_public_key_file.read_text(),
                "host_private_key": host_private_key_file.read_text(),
                "host_ip": host_ip,
            }
            host_public_key_file.unlink()
            host_private_key_file.unlink()
            public_key_file.unlink()
            private_key_file.unlink()
            yield RPCTaskResult(status=RPCTaskStatus.COMPLETED, response=json.dumps(response))
        else:
            yield RPCTaskResult(
                status=RPCTaskStatus.ERROR, response=f"Process exited with code {exit_code}\n: {stderr.decode('utf-8')}"
            )
