from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import Enum
from logging import Logger
from typing import Generator, TYPE_CHECKING, Optional

from mashumaro.mixins.json import DataClassJSONMixin

if TYPE_CHECKING:
    from addr_agent.agent_feature.agent_feature import AgentFeature


class RPCTaskStatus(Enum):
    CREATED = "created"
    ACQUIRED = "acquired"
    PROGRESS = "progress"
    COMPLETED = "completed"
    ERROR = "error"


@dataclass
class RPCTask(DataClassJSONMixin):
    uuid: str
    method: str
    request: str = ""


@dataclass
class RPCTaskResult(DataClassJSONMixin):
    status: RPCTaskStatus
    print_datetime: bool = True
    response: str = ""
    uuid: Optional[str] = None


class RPCMethodImpl(ABC):
    agent_feature: "AgentFeature"
    request: str = ""
    logger: Logger

    @abstractmethod
    async def run(self) -> Generator[RPCTaskResult, None, None]:
        ...
